@@ -8,10 +8,10 @@ use Params::Validate qw/:all/;
 use B qw/svref_2object/;
 use Exporter 'import';
 
-our @EXPORT = qw/render_me moon_test_one sunrise/;
+our @EXPORT = qw/render_me moon_test moon_test_one sunrise/;
 
 our %EXPORT_TAGS = (
-    all     => [qw/render_me moon_test_one sunrise/],
+    all     => [qw/render_me moon_test moon_test_one sunrise/],
     element => [qw/render_me sunrise/]
 );
 
@@ -24,11 +24,11 @@ Moonshine::Test - Test!
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
 =cut
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 =head1 SYNOPSIS
 
@@ -52,6 +52,8 @@ our $VERSION = '0.06';
 
 =over
 
+=item moon_test
+
 =item moon_test_one
 
 =item render_me
@@ -231,74 +233,145 @@ sub moon_test_one {
         }
         when ('ref_key_scalar') {
             return exists $instruction{key}
-                ? is( $test[0]->{$instruction{key}}, $expected[0], "$test_name is ref - has scalar key: $instruction{key} - is - $expected[0]")
-                : ok(0, "No key passed to test - ref_key_scalar - testing - $test_name");
+              ? is(
+                $test[0]->{ $instruction{key} },
+                $expected[0],
+"$test_name is ref - has scalar key: $instruction{key} - is - $expected[0]"
+              )
+              : ok(
+                0,
+                "No key passed to test - ref_key_scalar - testing - $test_name"
+              );
         }
         when ('ref_key_like') {
             return exists $instruction{key}
-                ? like( $test[0]->{$instruction{key}}, qr/$expected[0]/, "$test_name is ref - has scalar key: $instruction{key} - like - $expected[0]")
-                : ok(0, "No key passed to test - ref_key_like - testing - $test_name");
+              ? like(
+                $test[0]->{ $instruction{key} },
+                qr/$expected[0]/,
+"$test_name is ref - has scalar key: $instruction{key} - like - $expected[0]"
+              )
+              : ok( 0,
+                "No key passed to test - ref_key_like - testing - $test_name" );
         }
         when ('ref_key_ref') {
             return exists $instruction{key}
-                ? is_deeply( $test[0]->{$instruction{key}}, $expected[0],
-                        "$test_name is ref - has ref key: $instruction{key} - is_deeply - ref" )
-                : ok(0, "No key passed to test - ref_key_ref - testing - $test_name");
+              ? is_deeply(
+                $test[0]->{ $instruction{key} },
+                $expected[0],
+"$test_name is ref - has ref key: $instruction{key} - is_deeply - ref"
+              )
+              : ok( 0,
+                "No key passed to test - ref_key_ref - testing - $test_name" );
         }
         when ('ref_index_scalar') {
             return exists $instruction{index}
-                ? is( $test[0]->[$instruction{index}], $expected[0], "$test_name is ref - has scalar index: $instruction{index} - is - $expected[0]")
-                : ok(0, "No index passed to test - ref_index_scalar - testing - $test_name");
+              ? is(
+                $test[0]->[ $instruction{index} ],
+                $expected[0],
+"$test_name is ref - has scalar index: $instruction{index} - is - $expected[0]"
+              )
+              : ok(
+                0,
+"No index passed to test - ref_index_scalar - testing - $test_name"
+              );
         }
         when ('ref_index_ref') {
             return exists $instruction{index}
-                ? is_deeply( $test[0]->[$instruction{index}], $expected[0],
-                        "$test_name is ref - has ref index: $instruction{index} - is_deeply - ref" )
-                : ok(0, "No index passed to test - ref_index_ref - testing - $test_name");
+              ? is_deeply(
+                $test[0]->[ $instruction{index} ],
+                $expected[0],
+"$test_name is ref - has ref index: $instruction{index} - is_deeply - ref"
+              )
+              : ok(
+                0,
+                "No index passed to test - ref_index_ref - testing - $test_name"
+              );
         }
         when ('ref_index_like') {
             return exists $instruction{index}
-                ? like( $test[0]->[$instruction{index}], qr/$expected[0]/,
-                    "$test_name is ref - has scalar index: $instruction{index} - like - $expected[0]")
-                : ok(0, "No index passed to test - ref_index_like - testing - $test_name");
+              ? like(
+                $test[0]->[ $instruction{index} ],
+                qr/$expected[0]/,
+"$test_name is ref - has scalar index: $instruction{index} - like - $expected[0]"
+              )
+              : ok(
+                0,
+"No index passed to test - ref_index_like - testing - $test_name"
+              );
         }
         when ('list_index_scalar') {
             return exists $instruction{index}
-                ? is( $test[$instruction{index}], $expected[0], "$test_name is list - has scalar index: $instruction{index} - is - $expected[0]")
-                : ok(0, "No index passed to test - list_index_scalar - testing - $test_name");
+              ? is(
+                $test[ $instruction{index} ],
+                $expected[0],
+"$test_name is list - has scalar index: $instruction{index} - is - $expected[0]"
+              )
+              : ok(
+                0,
+"No index passed to test - list_index_scalar - testing - $test_name"
+              );
         }
         when ('list_index_ref') {
             return exists $instruction{index}
-                ? is_deeply( $test[$instruction{index}], $expected[0],
-                        "$test_name is list - has ref index: $instruction{index} - is_deeply - ref" )
-                : ok(0, "No index passed to test - list_index_ref - testing - $test_name");
+              ? is_deeply(
+                $test[ $instruction{index} ],
+                $expected[0],
+"$test_name is list - has ref index: $instruction{index} - is_deeply - ref"
+              )
+              : ok(
+                0,
+"No index passed to test - list_index_ref - testing - $test_name"
+              );
         }
         when ('list_index_like') {
             return exists $instruction{index}
-                ? like( $test[$instruction{index}], qr/$expected[0]/,
-                    "$test_name is list - has scalar index: $instruction{index} - like - $expected[0]")
-                : ok(0, "No index passed to test - list_index_like - testing - $test_name");
+              ? like(
+                $test[ $instruction{index} ],
+                qr/$expected[0]/,
+"$test_name is list - has scalar index: $instruction{index} - like - $expected[0]"
+              )
+              : ok(
+                0,
+"No index passed to test - list_index_like - testing - $test_name"
+              );
         }
         when ('list_key_scalar') {
             return exists $instruction{key}
-                ? is( {@test}->{$instruction{key}}, $expected[0], "$test_name is list - has scalar key: $instruction{key} - is - $expected[0]")
-                : ok(0, "No key passed to test - list_key_scalar - testing - $test_name");
+              ? is(
+                {@test}->{ $instruction{key} },
+                $expected[0],
+"$test_name is list - has scalar key: $instruction{key} - is - $expected[0]"
+              )
+              : ok(
+                0,
+                "No key passed to test - list_key_scalar - testing - $test_name"
+              );
         }
         when ('list_key_ref') {
             return exists $instruction{key}
-                ? is_deeply( {@test}->{$instruction{key}}, $expected[0],
-                        "$test_name is list - has ref key: $instruction{key} - is_deeply - ref" )
-                : ok(0, "No key passed to test - list_key_ref - testing - $test_name");
+              ? is_deeply(
+                {@test}->{ $instruction{key} },
+                $expected[0],
+"$test_name is list - has ref key: $instruction{key} - is_deeply - ref"
+              )
+              : ok( 0,
+                "No key passed to test - list_key_ref - testing - $test_name" );
         }
         when ('list_key_like') {
             return exists $instruction{key}
-                ? like( {@test}->{$instruction{key}}, qr/$expected[0]/,
-                    "$test_name is list - has scalar key: $instruction{key} - like - $expected[0]")
-                : ok(0, "No key passed to test - list_key_like - testing - $test_name");
+              ? like(
+                {@test}->{ $instruction{key} },
+                qr/$expected[0]/,
+"$test_name is list - has scalar key: $instruction{key} - like - $expected[0]"
+              )
+              : ok(
+                0,
+                "No key passed to test - list_key_like - testing - $test_name"
+              );
         }
         when ('scalar') {
-            return is( $test[0], $expected[0],
-                sprintf "%s is scalar - is - %s", $test_name, $expected[0]);
+            return is( $test[0], $expected[0], sprintf "%s is scalar - is - %s",
+                $test_name, $expected[0] );
         }
         when ('hash') {
             return is_deeply( {@test}, $expected[0],
@@ -309,7 +382,7 @@ sub moon_test_one {
                 "$test_name is array - reference - is_deeply" );
         }
         when ('obj') {
-            return is( blessed $test[0],
+            return isa_ok( $test[0],
                 $expected[0],
                 "$test_name is Object - blessed - is - $expected[0]" );
         }
@@ -318,13 +391,13 @@ sub moon_test_one {
                 "$test_name is like - $expected[0]" );
         }
         when ('true') {
-            return is($test[0], 1, "$test_name is true - 1");
+            return is( $test[0], 1, "$test_name is true - 1" );
         }
         when ('false') {
-            return is($test[0], 0, "$test_name is false - 0");
+            return is( $test[0], 0, "$test_name is false - 0" );
         }
         when ('undef') {
-            return is($test[0], undef, "$test_name is undef");
+            return is( $test[0], undef, "$test_name is undef" );
         }
         when ('render') {
             return render_me(
@@ -340,6 +413,166 @@ sub moon_test_one {
     }
 }
 
+=head2 moon_test
+ 
+    moon_test(
+        name => 'Checking Many Things'
+        build => {
+             class => 'Moonshine::Element', 
+             args => {
+                tag => 'p',
+                text => 'hello'
+             }
+        },
+        instructions => [
+            {
+                test => 'scalar',
+                func => 'tag',
+                expected => 'hello',
+            },
+           {
+                test => 'scalar',
+                action => 'text',
+                expected => 'hello',
+           },
+           { 
+                test => 'render'
+                expected => '<p>hello</p>'
+           },
+        ],
+    );
+
+=head3 name
+
+The tests name
+
+    name => 'I rule the world',
+
+
+=head3 instance
+
+    my $instance = My::Object->new();
+    instance => $instance,
+
+=head3 build
+
+Build an instance
+
+    build => {
+        class => 'My::Object',
+        args  => { },
+    },
+
+=head3 instructions
+
+    instructions => [
+        {
+            test => 'scalar',
+            func => 'tag',
+            expected => 'hello',
+        },
+        {
+            test => 'scalar',
+            action => 'text',
+            expected => 'hello',
+        },
+        { 
+            test => 'render'
+            expected => '<p>hello</p>'
+        },
+    ],
+
+=head3 subtest
+
+    instructions => [
+        {
+            test => 'obj',
+            func => 'glyphicon',
+            args => { switch => 'search' },
+            subtest => [
+                {
+                   test => 'scalar',
+                   func => 'class',
+                   expected => 'glyphicon glyphicon-search',
+                },
+                ...
+            ]
+        }
+    ]
+
+=cut
+
+sub moon_test {
+    my %instruction = validate_with(
+        params => \@_,
+        spec   => {
+            build        => { type => HASHREF, optional => 1, },
+            instance     => { type => HASHREF, optional => 1, },
+            instructions => { type => ARRAYREF },
+            name         => { type => SCALAR },
+        }
+    );
+
+    my $instance =
+      $instruction{build}
+      ? _build_me( $instruction{build} )
+      : $instruction{instance};
+
+    my %test_info = (
+        fail   => 0,
+        tested => 0,
+    );
+
+    foreach my $test ( @{ $instruction{instructions} } ) {
+        my $subtests = delete $test->{subtest};
+        $test_info{tested}++;
+        $test_info{fail}++
+          unless moon_test_one(
+            instance => $instance,
+            %{$test}
+          );
+
+        if ($subtests){
+            my $new_instance = _run_the_code({
+                instance => $instance, 
+                %{$test}
+            });
+            my $new_instructions = {
+                instance => $new_instance,
+                instructions => $subtests,
+                name => "Subtest -> $instruction{name}",
+            };
+            moon_test($new_instructions);
+        }
+    }
+
+    $test_info{ok} = $test_info{fail} ? 0 : 1;
+    return ok(
+        $test_info{ok},
+        sprintf(
+"moon_test: %s - tested %d instructions - success: %d - failure: %d",
+            $instruction{name},                        $test_info{tested},
+            ( $test_info{tested} - $test_info{fail} ), $test_info{fail},
+        )
+    );
+}
+
+sub _build_me {
+    my %instruction = validate_with(
+        params => \@_,
+        spec   => {
+            class => 1,
+            new   => { default => 'new' },
+            args  => { optional => 1, type => HASHREF },
+        }
+    );
+
+    my $new = $instruction{new};
+    return $instruction{args}
+      ? $instruction{class}->$new( $instruction{args} )
+      : $instruction{class}->$new;
+}
+
 =head2 render_me
 
 Test render directly on a Moonshine::Element.
@@ -406,7 +639,8 @@ sub _run_the_code {
         return ( $test_name, $instruction->{instance} );
     }
 
-    die('instruction passed to _run_the_code must have a func, meth or instance'
+    die(
+        'instruction passed to _run_the_code must have a func, meth or instance'
     );
 }
 
@@ -419,8 +653,7 @@ sub _run_the_code {
 sub sunrise {
     my $done_testing = done_testing(shift);
     diag explain $done_testing;
-    diag sprintf(
-        '
+    diag sprintf( '
                                    %s
             ^^                   @@@@@@@@@
        ^^       ^^            @@@@@@@@@@@@@@@
@@ -432,8 +665,7 @@ sub sunrise {
    -  --     -         -      ------  -- ---       -- - --  -- -
  -  -       - -      -           -- ------  -      --  -             --
        -             -        -      -      --   -             -',
-        shift // ' \o/ '
-    );
+        shift // ' \o/ ' );
     return $done_testing;
 }
 
